/*
 * Use TIMER_0, pre-configured to run with 1 kHz timeout, as system clock.
 *
 *  $Id: clock.c,v 1.2 2005/02/03 16:01:04 ericn Exp $
 */

#include <rtems.h>
#include <bsp.h>
#include <rtems/score/cpu.h>
#include <altera_avalon_timer_regs.h>

#define CLOCK_VECTOR TIMER_0_IRQ

/*
 * Periodic interval timer interrupt handler
 */
#define Clock_driver_support_at_tick()                     \
    do { /* Clear the TO bit in status reg. */             \
        IOWR_ALTERA_AVALON_TIMER_STATUS (TIMER_0_BASE, 0); \
    } while (0)                                            \

/*
 * Attach clock interrupt handler
 */
#define Clock_driver_support_install_isr( _new, _old )             \
    do {                                                           \
        _old = (rtems_isr_entry)set_vector(_new, TIMER_0_IRQ, 1);  \
    } while(0)

/*
 * Turn off the clock
 */
#define Clock_driver_support_shutdown_hardware()         \
    do {                                                 \
      IOWR_ALTERA_AVALON_TIMER_CONTROL (TIMER_0_BASE,    \
                ALTERA_AVALON_TIMER_CONTROL_STOP_MSK);   \
    } while(0)

/*
 * Set up the clock hardware
 */
void Clock_driver_support_initialize_hardware(void)
{
  uint32_t old_ie;
  uint32_t period;
  IOWR_ALTERA_AVALON_TIMER_CONTROL (TIMER_0_BASE,
    ALTERA_AVALON_TIMER_CONTROL_STOP_MSK);

  period = (TIMER_0_FREQ/1000000L)*BSP_Configuration.microseconds_per_tick - 1;
  IOWR_ALTERA_AVALON_TIMER_PERIODH(TIMER_0_BASE, period >> 16);
  IOWR_ALTERA_AVALON_TIMER_PERIODL(TIMER_0_BASE, period & 0xFFFF);

  IOWR_ALTERA_AVALON_TIMER_CONTROL (TIMER_0_BASE,
    ALTERA_AVALON_TIMER_CONTROL_ITO_MSK  |
    ALTERA_AVALON_TIMER_CONTROL_CONT_MSK |
    ALTERA_AVALON_TIMER_CONTROL_START_MSK);

  NIOS2_READ_IENABLE(old_ie);
  NIOS2_WRITE_IENABLE(old_ie | (1<<TIMER_0_IRQ)); 
}

#include "../../../shared/clockdrv_shell.c"

